@@ -1,11 +1,16 @@
 from __future__ import annotations
 
+import pathlib
 import re
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Pattern, Union
+from typing import Any, Dict, List, Optional, Pattern, Union
 
 import stringcase
-import yaml
-from datamodel_code_generator import DataModelField, snooper_to_methods
+from datamodel_code_generator import (
+    DataModelField,
+    cached_property,
+    load_yaml,
+    snooper_to_methods,
+)
 from datamodel_code_generator.imports import Import, Imports
 from datamodel_code_generator.parser.jsonschema import (
     JsonSchemaObject,
@@ -15,17 +20,7 @@ from datamodel_code_generator.parser.openapi import OpenAPIParser as OpenAPIMode
 from datamodel_code_generator.types import DataType
 from pydantic import BaseModel, root_validator
 
-if TYPE_CHECKING:
-    from typing import Any, Callable
-
-    cached_property: Callable[..., Any]
-else:
-    try:
-        from functools import cached_property
-    except ImportError:
-        # For Python3.7
-        from cached_property import cached_property
-
+MODEL_PATH: pathlib.Path = pathlib.Path("models.py")
 
 RE_APPLICATION_JSON_PATTERN: Pattern[str] = re.compile(r'^application/.*json$')
 
@@ -106,6 +101,7 @@ class Operation(CachedPropertyModel):
     type: UsefulStr
     path: UsefulStr
     operationId: Optional[UsefulStr]
+    summary: Optional[str]
     parameters: List[Dict[str, Any]] = []
     responses: Dict[UsefulStr, Any] = {}
     requestBody: Dict[str, Any] = {}
@@ -132,7 +128,7 @@ class Operation(CachedPropertyModel):
             for content_type, schema in requests.contents.items():
                 # TODO: support other content-types
                 if RE_APPLICATION_JSON_PATTERN.match(content_type):
-                    data_type = self.get_data_type(schema)
+                    data_type = self.get_data_type(schema, 'request')
                     arguments.append(
                         # TODO: support multiple body
                         Argument(
@@ -175,11 +171,12 @@ class Operation(CachedPropertyModel):
         for status_code, detail in self.responses.items():
             ref: Optional[str] = detail.get('$ref')
             if ref:
-                content = get_ref_body(
-                    ref, self.openapi_model_parser, self.components
-                ).get("content", {})
+                ref_body = get_ref_body(ref, self.openapi_model_parser, self.components)
+                content = ref_body.get("content", {})
+                description = ref_body.get("description")
             else:
                 content = detail.get("content", {})
+                description = detail.get("description")
             contents = {}
             for content_type, obj in content.items():
                 contents[content_type] = (
@@ -190,9 +187,7 @@ class Operation(CachedPropertyModel):
 
             responses.append(
                 Response(
-                    status_code=status_code,
-                    description=detail.get("description"),
-                    contents=contents,
+                    status_code=status_code, description=description, contents=contents,
                 )
             )
         return responses
@@ -234,13 +229,13 @@ class Operation(CachedPropertyModel):
             arguments.append(self.request)
         return arguments
 
-    def get_data_type(self, schema: JsonSchemaObject) -> DataType:
+    def get_data_type(self, schema: JsonSchemaObject, suffix: str = '') -> DataType:
         if schema.ref:
             data_type = self.openapi_model_parser.get_ref_data_type(schema.ref)
             data_type.imports_.append(
                 Import(
                     # TODO: Improve import statements
-                    from_=self.openapi_model_parser.current_source_path,  # model_path_var.get(),
+                    from_=f'.{MODEL_PATH.stem}',
                     import_=data_type.type,
                 )
             )
@@ -249,8 +244,23 @@ class Operation(CachedPropertyModel):
             # TODO: Improve handling array
             items = schema.items if isinstance(schema.items, list) else [schema.items]
             return self.openapi_model_parser.data_type(
-                data_types=[self.get_data_type(i) for i in items], is_list=True
+                data_types=[self.get_data_type(i, suffix) for i in items], is_list=True
             )
+        elif schema.is_object:
+            camelcase_path = stringcase.camelcase(self.path[1:].replace("/", "_"))
+            capitalized_suffix = suffix.capitalize()
+            name: str = f'{camelcase_path}{self.type.capitalize()}{capitalized_suffix}'
+            path = ['paths', self.path, self.type, capitalized_suffix]
+
+            data_model = self.openapi_model_parser.parse_object(name, schema, path)
+            data_type = self.openapi_model_parser.data_type.from_model_name(
+                data_model.name
+            )
+            self.imports.append(
+                Import(from_=f'.{MODEL_PATH.stem}', import_=data_type.type,)
+            )
+            return data_type
+
         return self.openapi_model_parser.get_data_type(schema)
 
     def get_parameter_type(
@@ -267,7 +277,7 @@ class Operation(CachedPropertyModel):
 
         field = DataModelField(
             name=name,
-            data_type=self.get_data_type(schema),
+            data_type=self.get_data_type(schema, 'parameter'),
             required=parameter.get("required") or parameter.get("in") == "path",
         )
         self.imports.extend(field.imports)
@@ -294,7 +304,7 @@ class Operation(CachedPropertyModel):
             if response.status_code.startswith("2"):
                 for content_type, schema in response.contents.items():
                     if RE_APPLICATION_JSON_PATTERN.match(content_type):
-                        data_type = self.get_data_type(schema)
+                        data_type = self.get_data_type(schema, 'response')
                         data_types.append(data_type)
                         self.imports.extend(data_type.imports_)
 
@@ -443,16 +453,19 @@ class ParsedObject:
 @snooper_to_methods(max_variable_length=None)
 class OpenAPIParser:
     def __init__(
-        self, input_name: str, input_text: str, model_path: Optional[str] = None
+        self,
+        input_name: str,
+        input_text: str,
+        openapi_model_parser: Optional[OpenAPIModelParser] = None,
     ) -> None:
         self.input_name: str = input_name
         self.input_text: str = input_text
-        self.openapi_model_parser: OpenAPIModelParser = OpenAPIModelParser(source='')
-        if model_path:
-            self.openapi_model_parser.current_source_path = '.models'
+        self.openapi_model_parser: OpenAPIModelParser = openapi_model_parser or OpenAPIModelParser(
+            source=''
+        )
 
     def parse(self) -> ParsedObject:
-        openapi = yaml.safe_load(self.input_text)
+        openapi = load_yaml(self.input_text)
         return self.parse_paths(openapi)
 
     def parse_security(
